@@ -1,8 +1,7 @@
 #!/bin/bash
 
-if [[ ! $(flatpak list|grep org.flatpak.Builder|wc -l) -ge 1 ]]; then 
-	echo Please ensure the flatpak builder is installed by running:
-	echo flatpak install flathub org.flatpak.Builder
+if [[ ! -x $(which flatpak-builder) ]]; then
+	echo Please install flatpak-builder
 	exit
 fi
 
@@ -12,7 +11,7 @@ if [[ ! $(flatpak list|grep org.freedesktop.Sdk|wc -l) -ge 1 ]]; then
 	exit
 fi
 
-flatpak run org.flatpak.Builder --repo=myrepo --force-clean build-flatpak contrib/linux/com.dosbox_x.DOSBox-X.yaml
+flatpak-builder --repo=myrepo --force-clean build-flatpak contrib/linux/com.dosbox_x.DOSBox-X.yaml
 if [ $? -eq 0 ]; then
 	echo
 	echo You can now install the flatpak by running the following commands:
@@ -24,5 +23,5 @@ if [ $? -eq 0 ]; then
 	echo  flatpak run com.dosbox_x.DOSBox-X
 	echo
 	echo Or you can test it without installing by running:
-	echo  flatpak run org.flatpak.Builder --run build-flatpak contrib/linux/com.dosbox_x.DOSBox-X.yaml dosbox-x
+	echo  flatpak-builder --run build-flatpak contrib/linux/com.dosbox_x.DOSBox-X.yaml dosbox-x
 fi
