@@ -135,7 +135,7 @@ void DOS_Drive_Cache::SetBaseDir(const char* baseDir, DOS_Drive *drive) {
     if (strlen(baseDir) == 0) return;
 
     uint16_t id;
-    strcpy(basePath,baseDir);
+    if (basePath != baseDir) strcpy(basePath,baseDir); /* NTS: pointer check because Valgrind says this was called with basePath == baseDir */
     this->drive = drive;
     if (OpenDir(baseDir,id)) {
         char* result = 0, *lresult = 0;
@@ -531,28 +531,27 @@ Bits DOS_Drive_Cache::GetLongName(CFileInfo* curDir, char* shortName) {
     RemoveTrailingDot(shortName);
     // Search long name and return array number of element
     Bits res;
-    if (!uselfn) {
-        Bits low	= 0;
-        Bits high	= (Bits)(filelist_size-1);
-        Bits mid;
-        while (low<=high) {
-            mid = (low+high)/2;
-            res = strcmp(shortName,curDir->fileList[mid]->shortname);
-            if (res>0)	low  = mid+1; else
-            if (res<0)	high = mid-1; else
-            {	// Found
-                strcpy(shortName,curDir->fileList[mid]->orgname);
-                return mid;
-            };
-        }
-    } else if (strlen(shortName)) {
-        for (Bitu i=0; i<filelist_size; i++) {
-            if (!strcasecmp(shortName,curDir->fileList[i]->orgname) || !strcasecmp(shortName,curDir->fileList[i]->shortname)) {
-                strcpy(shortName,curDir->fileList[i]->orgname);
-                return (Bits)i;
-            }
-        }
-    }
+	Bits low	= 0;
+	Bits high	= (Bits)(filelist_size-1);
+	Bits mid;
+	while (low<=high) {
+		mid = (low+high)/2;
+		res = strcmp(shortName,curDir->fileList[mid]->shortname);
+		if (res>0)	low  = mid+1; else
+		if (res<0)	high = mid-1; else
+		{	// Found
+			strcpy(shortName,curDir->fileList[mid]->orgname);
+			return mid;
+		};
+	}
+	if (uselfn && strlen(shortName)) {
+		for (Bitu i=0; i<filelist_size; i++) {
+			if (!strcasecmp(shortName,curDir->fileList[i]->orgname)) {
+				strcpy(shortName,curDir->fileList[i]->orgname);
+				return (Bits)i;
+		    }
+		}
+	}
 
 #ifdef WINE_DRIVE_SUPPORT
     if (strlen(shortName) < 8 || shortName[4] != '~' || shortName[5] == '.' || shortName[6] == '.' || shortName[7] == '.') return -1; // not available
