@@ -148,6 +148,7 @@ PC98_GDC_state::PC98_GDC_state() {
     vertical_back_porch_width = 0;
     reset_fifo();
     reset_rfifo();
+    draw_reset();
 }
 
 size_t PC98_GDC_state::fifo_can_read(void) {
@@ -247,17 +248,31 @@ void PC98_GDC_state::take_cursor_pos(unsigned char bi) {
      *   0 = [3:2] = 0
      *   EAD(H) = [1:0] = address[17:16] */
     if (bi == 1) {
-		vga.config.cursor_start &= ~(0xFFu << 0u);
-		vga.config.cursor_start |=  cmd_parm_tmp[0] << 0u;
+        if(master_sync) {
+            vga.config.cursor_start &= ~(0xFFu << 0u);
+            vga.config.cursor_start |=  cmd_parm_tmp[0] << 0u;
+        } else {
+            draw.ead = cmd_parm_tmp[0];
+        }
     }
     else if (bi == 2) {
-		vga.config.cursor_start &= ~(0xFFu << 8u);
-		vga.config.cursor_start |=  (unsigned int)cmd_parm_tmp[1] << 8u;
+        if(master_sync) {
+            vga.config.cursor_start &= ~(0xFFu << 8u);
+            vga.config.cursor_start |=  (unsigned int)cmd_parm_tmp[1] << 8u;
+        } else {
+            draw.ead |= (uint32_t)cmd_parm_tmp[1] << 8;
+        }
     }
     else if (bi == 3) {
-		vga.config.cursor_start &= ~(0x03u << 16u);
-		vga.config.cursor_start |=  (cmd_parm_tmp[2] & 3u) << 16u;
-
+        if(master_sync) {
+            vga.config.cursor_start &= ~(0x03u << 16u);
+            vga.config.cursor_start |=  (cmd_parm_tmp[2] & 3u) << 16u;
+        } else {
+            draw.ead |= (uint32_t)(cmd_parm_tmp[2] & 0x03) << 16;
+            draw.dad = (uint8_t)(cmd_parm_tmp[2] >> 4);
+            draw.base = gram_base[(draw.ead >> 14) & 3];
+            draw.wg = cmd_parm_tmp[2] & 0x08;
+        }
         // TODO: "dot address within the word"
     }
 }
@@ -322,6 +337,7 @@ void PC98_GDC_state::idle_proc(void) {
                 idle = true;
                 reset_fifo();
                 reset_rfifo();
+                draw_reset();
                 break;
             case GDC_CMD_DISPLAY_BLANK:  // 0x0C   0 0 0 0 1 1 0 DE
             case GDC_CMD_DISPLAY_BLANK+1:// 0x0D   DE=display enable
@@ -334,6 +350,14 @@ void PC98_GDC_state::idle_proc(void) {
                 current_command &= ~1;
                 LOG_MSG("GDC: sync");
                 break;
+            case GDC_CMD_MODE_REPLACE:        // 0x20        0 0 1 0 0 0 0 0
+            case GDC_CMD_MODE_COMPLEMENT:     // 0x21        0 0 1 0 0 0 0 1
+            case GDC_CMD_MODE_CLEAR:          // 0x22        0 0 1 0 0 0 1 0
+            case GDC_CMD_MODE_SET:            // 0x23        0 0 1 0 0 0 1 1
+                draw.mode = current_command & 0x03;
+                break;
+            case GDC_CMD_ZOOM:                // 0x46        0 1 0 0 0 1 1 0
+                break;
             case GDC_CMD_PITCH_SPEC:          // 0x47        0 1 0 0 0 1 1 1
                 break;
             case GDC_CMD_CURSOR_POSITION:     // 0x49        0 1 0 0 1 0 0 1
@@ -342,6 +366,14 @@ void PC98_GDC_state::idle_proc(void) {
             case GDC_CMD_CURSOR_CHAR_SETUP:   // 0x4B        0 1 0 0 1 0 1 1
 //              LOG_MSG("GDC: cursor setup");
                 break;
+            case GDC_CMD_VECTW:
+                break;
+            case GDC_CMD_TEXTE:               // 0x68        0 1 1 0 1 0 0 0
+            case GDC_CMD_VECTE:               // 0x6C        0 1 1 0 1 1 0 0
+                if(!master_sync) {
+                    exec(current_command);
+                }
+                break;
             case GDC_CMD_START_DISPLAY:       // 0x6B        0 1 1 0 1 0 1 1
                 display_enable = true;
                 idle = false;
@@ -396,6 +428,12 @@ void PC98_GDC_state::idle_proc(void) {
                     }
                 }
                 break;
+            case GDC_CMD_ZOOM:
+                if(proc_step < 1) {
+                    draw.zoom = (uint8_t)(val & 0x0f);
+                    proc_step++;
+                }
+                break;
             case GDC_CMD_PITCH_SPEC:
                 if (proc_step < 1)
                     display_pitch = (val != 0) ? val : 0x100;
@@ -406,6 +444,12 @@ void PC98_GDC_state::idle_proc(void) {
                     take_cursor_pos(proc_step);
                 }
                 break;
+            case GDC_CMD_VECTW:
+                if(proc_step < 11) {
+                    cmd_parm_tmp[proc_step++] = (uint8_t)val;
+                    vectw(proc_step);
+                }
+                break;
             case GDC_CMD_CURSOR_CHAR_SETUP:
                 if (proc_step < 3) {
                     cmd_parm_tmp[proc_step++] = (uint8_t)val;
@@ -415,6 +459,15 @@ void PC98_GDC_state::idle_proc(void) {
                 }
                 break;
             case GDC_CMD_PARAMETER_RAM_LOAD:
+                /* Parameter bytes 8 and 9 hold the 16-bit pattern for graphics.
+                 * nanshiki's original code seems to imply that somehow the hardware
+                 * supports either a 64-bit pattern across the 8 bytes or 4 16-bit
+                 * patterns across the same. Given that a few PC-9821 games exploit
+                 * a bug in older hardware to get 4 display partitions in graphics,
+                 * that wouldn't surprise me.
+                 *
+                 * PC-98 Windows 3.1 requires us to do this or else lines do not draw properly. */
+                if (param_ram_wptr >= 8 && param_ram_wptr <= 15) draw.tx[param_ram_wptr-8] = (uint8_t)val;
                 param_ram[param_ram_wptr] = (uint8_t)val;
                 if ((++param_ram_wptr) >= 16) param_ram_wptr = 0;
                 break;
