@@ -96,6 +96,7 @@ static unsigned long ttfSize = sizeof(DOSBoxTTFbi), ttfSizeb = 0, ttfSizei = 0,
 static void * ttfFont = DOSBoxTTFbi, * ttfFontb = NULL, * ttfFonti = NULL, * ttfFontbi = NULL;
 extern int posx, posy, eurAscii, NonUserResizeCounter;
 extern bool rtl, gbk, chinasea, switchttf, force_conversion, blinking, showdbcs, loadlang, window_was_maximized;
+extern const char* RunningProgram;
 extern uint8_t ccount;
 extern uint16_t cpMap[512], cpMap_PC98[256];
 uint16_t cpMap_copy[256];
@@ -523,7 +524,8 @@ int setTTFCodePage() {
             }
         if (eurAscii != -1 && TTF_GlyphIsProvided(ttf.SDL_font, 0x20ac))
             cpMap[eurAscii] = 0x20ac;
-        initcodepagefont();
+        if (strcmp(RunningProgram, "LOADLIN") && !dos_kernel_disabled)
+            initcodepagefont();
 #if defined(WIN32) && !defined(HX_DOS)
         DOSBox_SetSysMenu();
 #endif
@@ -726,8 +728,8 @@ void OUTPUT_TTF_Select(int fsize) {
                 c=80;
                 r=real_readb(0x60,0x113) & 0x01 ? 25 : 20;
             } else {
-                c=real_readw(BIOSMEM_SEG,BIOSMEM_NB_COLS);
-                r=(uint16_t)(IS_EGAVGA_ARCH?real_readb(BIOSMEM_SEG,BIOSMEM_NB_ROWS):24)+1;
+                c=strcmp(RunningProgram, "LOADLIN")?real_readw(BIOSMEM_SEG,BIOSMEM_NB_COLS):80;
+                r=(uint16_t)(IS_EGAVGA_ARCH&&strcmp(RunningProgram, "LOADLIN")?real_readb(BIOSMEM_SEG,BIOSMEM_NB_ROWS):24)+1;
             }
             if (ttf.lins<1||ttf.cols<1)	{
                 if (ttf.cols<1)
@@ -763,8 +765,8 @@ void OUTPUT_TTF_Select(int fsize) {
                 c=80;
                 r=real_readb(0x60,0x113) & 0x01 ? 25 : 20;
             } else {
-                c=real_readw(BIOSMEM_SEG,BIOSMEM_NB_COLS);
-                r=(uint16_t)(IS_EGAVGA_ARCH?real_readb(BIOSMEM_SEG,BIOSMEM_NB_ROWS):24)+1;
+                c=strcmp(RunningProgram, "LOADLIN")?real_readw(BIOSMEM_SEG,BIOSMEM_NB_COLS):80;
+                r=(uint16_t)(IS_EGAVGA_ARCH&&strcmp(RunningProgram, "LOADLIN")?real_readb(BIOSMEM_SEG,BIOSMEM_NB_ROWS):24)+1;
             }
             ttf.cols=c;
             ttf.lins=r;
@@ -1345,11 +1347,13 @@ void ttf_setlines(int cols, int lins) {
 
 void ttf_switch_on(bool ss=true) {
     if ((ss&&ttfswitch)||(!ss&&switch_output_from_ttf)) {
-        uint16_t oldax=reg_ax;
-        reg_ax=0x1600;
-        CALLBACK_RunRealInt(0x2F);
-        if (reg_al!=0&&reg_al!=0x80) {reg_ax=oldax;return;}
-        reg_ax=oldax;
+        if (strcmp(RunningProgram, "LOADLIN") && !dos_kernel_disabled) {
+            uint16_t oldax=reg_ax;
+            reg_ax=0x1600;
+            CALLBACK_RunRealInt(0x2F);
+            if (reg_al!=0&&reg_al!=0x80) {reg_ax=oldax;return;}
+            reg_ax=oldax;
+        }
         if (window_was_maximized&&!GFX_IsFullscreen()) {
 #if defined(WIN32)
             ShowWindow(GetHWND(), SW_RESTORE);
@@ -1362,6 +1366,7 @@ void ttf_switch_on(bool ss=true) {
         SetVal("sdl", "output", "ttf");
         std::string showdbcsstr = static_cast<Section_prop *>(control->GetSection("dosv"))->Get_string("showdbcsnodosv");
         showdbcs = showdbcsstr=="true"||showdbcsstr=="1"||(showdbcsstr=="auto" && (loadlang || dbcs_sbcs));
+        if (!IS_EGAVGA_ARCH) showdbcs = false;
         void OutputSettingMenuUpdate(void);
         OutputSettingMenuUpdate();
         if (ss) ttfswitch = false;
@@ -1416,7 +1421,7 @@ void ttf_switch_off(bool ss=true) {
 #endif
         }
         KEYBOARD_Clear();
-        showdbcs = true;
+        showdbcs = IS_EGAVGA_ARCH;
         change_output(out);
         SetVal("sdl", "output", output);
         void OutputSettingMenuUpdate(void);
